import time
from datetime import datetime

from classic.components import component
from classic.scheduler import Scheduler, by_cron, by_period


@component
class SomeComponent:
    periodic_value: datetime = None
    cron_value: datetime = None

    @by_period(1)
    def periodic(self):
        self.periodic_value = datetime.now()

    @by_cron('* * * * * *')
    def cron(self):
        self.cron_value = datetime.now()


def test_autoregister(scheduler: Scheduler) -> None:
    obj = SomeComponent(scheduler=scheduler)

    time.sleep(2)

    assert obj.scheduler is scheduler
    assert obj.periodic_value is not None
    assert obj.cron_value is not None



from datetime import timedelta
from typing import TypeVar

from classic.components import add_extra_annotation

from .scheduler import Scheduler


Method = TypeVar('Method')
PeriodValue = int | float | timedelta


def by_cron(schedule: str | list[str]) -> Method:
    """
    Помечает метод для автоматической регистрации в планировщике.
    Планировщик будет выполнять метод с указанным расписанием.
    Расписание можно указать одно, а можно сразу несколько.
    """

    def decorator(fn: Method):
        fn.__by_cron__ = [schedule] if isinstance(schedule, str) else schedule
        add_extra_annotation(fn, 'scheduler', Scheduler)
        return fn
    return decorator



def by_period(period: PeriodValue | list[PeriodValue]) -> Method:
    """
    Помечает метод для автоматической регистрации в планировщике.
    Планировщик будет выполнять метод с указанным периодом.
    Период можно указать один, а можно сразу несколько.
    """

    def decorator(fn: Method):
        fn.__by_period__ = (
            [period] if isinstance(period, (int, float, timedelta)) else period
        )
        add_extra_annotation(fn, 'scheduler', Scheduler)
        return fn

    return decorator
